@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: qd_base.h,v 1.11 2003/11/07 22:08:17 leonb Exp $
-// $Name: release_3_5_15 $
+// $Id: qd_base.h,v 1.12 2005/09/16 15:57:09 leonb Exp $
+// $Name: release_3_5_16 $
 
 #ifndef HDR_QD_BASE
 #define HDR_QD_BASE
@@ -112,7 +112,7 @@ protected:
      if (qrect.contains(pnt)) tip(qrect, text);
    }
 public:
-   QDMapAreaTip(const char * _text, const QRect & _qrect, QWidget * _parent) :
+   QDMapAreaTip(QString _text, const QRect & _qrect, QWidget * _parent) :
 	 QToolTip(_parent), text(_text), qrect(_qrect) {}
    virtual ~QDMapAreaTip(void) { clear(); }
 };
