@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: qd_base_events.cpp,v 1.15 2003/11/07 22:08:17 leonb Exp $
-// $Name: release_3_5_15 $
+// $Id: qd_base_events.cpp,v 1.16 2005/09/16 14:51:50 leonb Exp $
+// $Name: release_3_5_16 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -455,9 +455,10 @@ QDBase::processMouseMoveEvent(QMouseEvent * ev)
 	       GRect brect=new_cur_map_area->get_bound_rect();
 	       ma_mapper.map(brect);
 	       GUTF8String tip=url;
-	       if (comment.length()) tip=comment;
+	       if (comment.length()) 
+                 tip=comment;
 	       if (tip.length())
-		  map_area_tip=new QDMapAreaTip(tip, G2Q(brect), pane);
+                 map_area_tip=new QDMapAreaTip(QStringFromGString(tip), G2Q(brect), pane);
 	    }
 	 } else showStatus(" ");
 
