@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: DjVuDocument.cpp,v 1.11 2003/11/07 22:08:20 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: DjVuDocument.cpp,v 1.14 2005/08/07 21:32:27 leonb Exp $
+// $Name: release_3_5_16 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -64,6 +64,7 @@
 #include "DjVuDocument.h"
 #include "DjVmDoc.h"
 #include "DjVmDir0.h"
+#include "DjVmNav.h"
 #include "DjVuNavDir.h"
 #include "DjVuImage.h"
 #include "DjVuFileCache.h"
@@ -323,127 +324,135 @@ DjVuDocument::init_thread(void)
    }
    if (chkid=="FORM:DJVM")
    {
-      DEBUG_MSG("Got DJVM document here\n");
-      DEBUG_MAKE_INDENT(3);
-
-      size=iff.get_chunk(chkid);
-      if (chkid=="DIRM")
-      {
+     DEBUG_MSG("Got DJVM document here\n");
+     DEBUG_MAKE_INDENT(3);
+     
+     size=iff.get_chunk(chkid);
+     if (chkid=="DIRM")
+       {
 	 djvm_dir=DjVmDir::create();
 	 djvm_dir->decode(iff.get_bytestream());
 	 iff.close_chunk();
 	 if (djvm_dir->is_bundled())
-	 {
-	    DEBUG_MSG("Got BUNDLED file.\n");
-	    doc_type=BUNDLED;
-	 } else
-	 {
-	    DEBUG_MSG("Got INDIRECT file.\n");
-	    doc_type=INDIRECT;
-	 }
+           {
+             DEBUG_MSG("Got BUNDLED file.\n");
+             doc_type=BUNDLED;
+           } 
+         else
+           {
+             DEBUG_MSG("Got INDIRECT file.\n");
+             doc_type=INDIRECT;
+           }
 	 flags|=DOC_TYPE_KNOWN | DOC_DIR_KNOWN;
 	 pcaster->notify_doc_flags_changed(this, DOC_TYPE_KNOWN | DOC_DIR_KNOWN, 0);
 	 check_unnamed_files();
-      } else if (chkid=="DIR0")
-      {
+         
+         /* Check for NAVM */
+         size=iff.get_chunk(chkid);
+         if (size && chkid=="NAVM")
+           {
+             djvm_nav=DjVmNav::create();
+             djvm_nav->decode(iff.get_bytestream());
+             iff.close_chunk();
+           }
+       }
+     else if (chkid=="DIR0")
+       {
 	 DEBUG_MSG("Got OLD_BUNDLED file.\n");
 	 doc_type=OLD_BUNDLED;
 	 flags|=DOC_TYPE_KNOWN;
 	 pcaster->notify_doc_flags_changed(this, DOC_TYPE_KNOWN, 0);
 	 check_unnamed_files();
-      } else G_THROW( ERR_MSG("DjVuDocument.bad_format") );
-
-      if (doc_type==OLD_BUNDLED)
-      {
-	    // Read the DjVmDir0 directory. We are unable to tell what
-	    // files are pages and what are included at this point.
-	    // We only know that the first file with DJVU (BM44 or PM44)
-	    // form *is* the first page. The rest will become known
-	    // after we decode DjVuNavDir
+       } 
+     else 
+       G_THROW( ERR_MSG("DjVuDocument.bad_format") );
+     
+     if (doc_type==OLD_BUNDLED)
+       {
+         // Read the DjVmDir0 directory. We are unable to tell what
+         // files are pages and what are included at this point.
+         // We only know that the first file with DJVU (BM44 or PM44)
+         // form *is* the first page. The rest will become known
+         // after we decode DjVuNavDir
 	 djvm_dir0=DjVmDir0::create();
 	 djvm_dir0->decode(*iff.get_bytestream());
 	 iff.close_chunk();
-
-	    // Get offset to the first DJVU, PM44 or BM44 chunk
+         // Get offset to the first DJVU, PM44 or BM44 chunk
 	 int first_page_offset=0;
 	 while(!first_page_offset)
-	 {
-	    int offset;
-	    size=iff.get_chunk(chkid, &offset);
-	    if (size==0) G_THROW( ERR_MSG("DjVuDocument.no_page") );
-	    if (chkid=="FORM:DJVU" || chkid=="FORM:PM44" || chkid=="FORM:BM44")
-	    {
-	       DEBUG_MSG("Got 1st page offset=" << offset << "\n");
-	       first_page_offset=offset;
-	    }
-	    iff.close_chunk();
-	 }
-
-	    // Now get the name of this file
+           {
+             int offset;
+             size=iff.get_chunk(chkid, &offset);
+             if (size==0) G_THROW( ERR_MSG("DjVuDocument.no_page") );
+             if (chkid=="FORM:DJVU" || chkid=="FORM:PM44" || chkid=="FORM:BM44")
+               {
+                 DEBUG_MSG("Got 1st page offset=" << offset << "\n");
+                 first_page_offset=offset;
+               }
+             iff.close_chunk();
+           }
+         
+         // Now get the name of this file
 	 int file_num;
 	 for(file_num=0;file_num<djvm_dir0->get_files_num();file_num++)
-	 {
-	    DjVmDir0::FileRec & file=*djvm_dir0->get_file(file_num);
-	    if (file.offset==first_page_offset)
-	    {
-	       first_page_name=file.name;
-	       break;
-	    }
-	 }
+           {
+             DjVmDir0::FileRec & file=*djvm_dir0->get_file(file_num);
+             if (file.offset==first_page_offset)
+               {
+                 first_page_name=file.name;
+                 break;
+               }
+           }
 	 if (!first_page_name.length())
-	    G_THROW( ERR_MSG("DjVuDocument.no_page") );
-
+           G_THROW( ERR_MSG("DjVuDocument.no_page") );
 	 flags|=DOC_DIR_KNOWN;
 	 pcaster->notify_doc_flags_changed(this, DOC_DIR_KNOWN, 0);
 	 check_unnamed_files();
-      }
-   } else // chkid!="FORM:DJVM"
-   {
-	 // DJVU format
-      DEBUG_MSG("Got DJVU OLD_INDEXED or SINGLE_PAGE document here.\n");
-      doc_type=SINGLE_PAGE;
-
-      flags|=DOC_TYPE_KNOWN;
-      pcaster->notify_doc_flags_changed(this, DOC_TYPE_KNOWN, 0);
-      check_unnamed_files();
-   }
-
+       }
+   } 
+   else // chkid!="FORM:DJVM"
+     {
+       // DJVU format
+       DEBUG_MSG("Got DJVU OLD_INDEXED or SINGLE_PAGE document here.\n");
+       doc_type=SINGLE_PAGE;
+       flags|=DOC_TYPE_KNOWN;
+       pcaster->notify_doc_flags_changed(this, DOC_TYPE_KNOWN, 0);
+       check_unnamed_files();
+     }
    if (doc_type==OLD_BUNDLED || doc_type==SINGLE_PAGE)
-   {
-      DEBUG_MSG("Searching for NDIR chunks...\n");
-      ndir_file=get_djvu_file(-1);
-      if (ndir_file) ndir=ndir_file->decode_ndir();
-      ndir_file=0;	// Otherwise ~DjVuDocument() will stop (=kill) it
-      if (!ndir)
-      {
-	    // Seems to be 1-page old-style document. Create dummy NDIR
-	 if (doc_type==OLD_BUNDLED)
-	 {
-		 ndir=DjVuNavDir::create(GURL::UTF8("directory",init_url));
-	    ndir->insert_page(-1, first_page_name);
-	 } else
-	 {
-		 ndir=DjVuNavDir::create(GURL::UTF8("directory",init_url.base()));
-	    ndir->insert_page(-1, init_url.fname());
-	 }
-      } 
-      else
-      {
-	 if (doc_type==SINGLE_PAGE)
-	    doc_type=OLD_INDEXED;
-      }
-
-      flags|=DOC_NDIR_KNOWN;
-      pcaster->notify_doc_flags_changed(this, DOC_NDIR_KNOWN, 0);
-      check_unnamed_files();
-   }
-
+     {
+       DEBUG_MSG("Searching for NDIR chunks...\n");
+       ndir_file=get_djvu_file(-1);
+       if (ndir_file) ndir=ndir_file->decode_ndir();
+       ndir_file=0;	// Otherwise ~DjVuDocument() will stop (=kill) it
+       if (!ndir)
+         {
+           // Seems to be 1-page old-style document. Create dummy NDIR
+           if (doc_type==OLD_BUNDLED)
+             {
+               ndir=DjVuNavDir::create(GURL::UTF8("directory",init_url));
+               ndir->insert_page(-1, first_page_name);
+             } 
+           else
+             {
+               ndir=DjVuNavDir::create(GURL::UTF8("directory",init_url.base()));
+               ndir->insert_page(-1, init_url.fname());
+             }
+         } 
+       else
+         {
+           if (doc_type==SINGLE_PAGE)
+             doc_type=OLD_INDEXED;
+         }
+       flags|=DOC_NDIR_KNOWN;
+       pcaster->notify_doc_flags_changed(this, DOC_NDIR_KNOWN, 0);
+       check_unnamed_files();
+     }
+   
    flags|=DOC_INIT_OK;
    pcaster->notify_doc_flags_changed(this, DOC_INIT_OK, 0);
    check_unnamed_files();
-
    init_thread_flags|=FINISHED;
-   
    DEBUG_MSG("DOCUMENT IS FULLY INITIALIZED now: doc_type='" <<
 	     (doc_type==BUNDLED ? "BUNDLED" :
 	      doc_type==OLD_BUNDLED ? "OLD_BUNDLED" :
@@ -1402,11 +1411,8 @@ DjVuDocument::notify_file_flags_changed(const DjVuFile * source,
           can_compress_flag=true;
         }
       }
-      process_threqs();
    }
-   
-   if (set_mask & DjVuFile::DATA_PRESENT)
-      process_threqs();		// May be we can extract thumbnails now
+   process_threqs();
 }
 
 GP<DjVuFile>
@@ -1666,63 +1672,71 @@ DjVuDocument::get_djvm_doc()
    GP<DjVmDoc> doc=DjVmDoc::create();
 
    if (doc_type==BUNDLED || doc_type==INDIRECT)
-   {
-      DEBUG_MSG("Trivial: the document is either INDIRECT or BUNDLED: follow DjVmDir.\n");
-
-      GPList<DjVmDir::File> files_list=djvm_dir->get_files_list();
-      for(GPosition pos=files_list;pos;++pos)
-      {
-	 GP<DjVmDir::File> f=new DjVmDir::File(*files_list[pos]);
-	 GP<DjVuFile> file=url_to_file(id_to_url(f->get_load_name()));
-	 GP<DataPool> data;
-	 if (file->is_modified()) data=file->get_djvu_data(false);
-	 else data=file->get_init_data_pool();
-	 doc->insert_file(f, data);
-      }
-   } else if (doc_type==SINGLE_PAGE)
-   {
-      DEBUG_MSG("Creating: djvm for a single page document.\n");
-      GMap<GURL, void *> map_add;
-      GP<DjVuFile> file=get_djvu_file(0);
-      add_file_to_djvm(file, true, *doc, map_add,needs_compression_flag,can_compress_flag);
-   } else
-   {
-      DEBUG_MSG("Converting: the document is in an old format.\n");
-
-      GMap<GURL, void *> map_add;
-      if(recover_errors == ABORT)
-      {
-        for(int page_num=0;page_num<ndir->get_pages_num();page_num++)
-        {
-          GP<DjVuFile> file=url_to_file(ndir->page_to_url(page_num));
-          add_file_to_djvm(file, true, *doc, map_add,needs_compression_flag,can_compress_flag);
-        }
-      }else
-      {
-        for(int page_num=0;page_num<ndir->get_pages_num();page_num++)
-        {
-          G_TRY
-          {
-            GP<DjVuFile> file=url_to_file(ndir->page_to_url(page_num));
-            add_file_to_djvm(file, true, *doc, map_add,needs_compression_flag,can_compress_flag);
-          }
-          G_CATCH(ex)
-          {
-            G_TRY { 
-              get_portcaster()->notify_error(this, ex.get_cause());
-              GUTF8String emsg = ERR_MSG("DjVuDocument.skip_page") "\t" + (page_num+1);
-              get_portcaster()->notify_error(this, emsg);
-            }
-            G_CATCH_ALL
-            {
-              G_RETHROW;
-            }
-            G_ENDCATCH;
-          }
-          G_ENDCATCH;
-        }
-      }
-   }
+     {
+       GPList<DjVmDir::File> files_list=djvm_dir->get_files_list();
+       for(GPosition pos=files_list;pos;++pos)
+         {
+           GP<DjVmDir::File> f=new DjVmDir::File(*files_list[pos]);
+           GP<DjVuFile> file=url_to_file(id_to_url(f->get_load_name()));
+           GP<DataPool> data;
+           if (file->is_modified()) 
+             data=file->get_djvu_data(false);
+           else 
+             data=file->get_init_data_pool();
+           doc->insert_file(f, data);
+         }
+       if (djvm_nav)
+         doc->set_djvm_nav(djvm_nav);
+     } 
+   else if (doc_type==SINGLE_PAGE)
+     {
+       DEBUG_MSG("Creating: djvm for a single page document.\n");
+       GMap<GURL, void *> map_add;
+       GP<DjVuFile> file=get_djvu_file(0);
+       add_file_to_djvm(file, true, *doc, map_add,
+                        needs_compression_flag,can_compress_flag);
+     } 
+   else
+     {
+       DEBUG_MSG("Converting: the document is in an old format.\n");
+       GMap<GURL, void *> map_add;
+       if(recover_errors == ABORT)
+         {
+           for(int page_num=0;page_num<ndir->get_pages_num();page_num++)
+             {
+               GP<DjVuFile> file=url_to_file(ndir->page_to_url(page_num));
+               add_file_to_djvm(file, true, *doc, map_add,
+                                needs_compression_flag,can_compress_flag);
+             }
+         }
+       else
+         {
+           for(int page_num=0;page_num<ndir->get_pages_num();page_num++)
+             {
+               G_TRY
+                 {
+                   GP<DjVuFile> file=url_to_file(ndir->page_to_url(page_num));
+                   add_file_to_djvm(file, true, *doc, map_add,
+                                    needs_compression_flag,can_compress_flag);
+                 }
+               G_CATCH(ex)
+                 {
+                   G_TRY { 
+                     get_portcaster()->notify_error(this, ex.get_cause());
+                     GUTF8String emsg = ERR_MSG("DjVuDocument.skip_page") "\t" 
+                                      + (page_num+1);
+                     get_portcaster()->notify_error(this, emsg);
+                   }
+                   G_CATCH_ALL
+                     {
+                       G_RETHROW;
+                     }
+                   G_ENDCATCH;
+                 }
+               G_ENDCATCH;
+             }
+         }
+     }
    return doc;
 }
 
