@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: DjVuMessage.cpp,v 1.14 2004/05/05 15:12:42 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: DjVuMessage.cpp,v 1.17 2005/08/07 21:32:28 leonb Exp $
+// $Name: release_3_5_16 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -84,9 +84,7 @@
 # include <pwd.h>
 # include <sys/types.h>
 #endif
-#ifndef UNDER_CE
-# include <locale.h>
-#endif
+#include <locale.h>
 #ifndef LC_MESSAGES
 # define LC_MESSAGES LC_ALL
 #endif
@@ -108,8 +106,6 @@ DjVuMessage::programname(void)
 }
 
 static const char namestring[]="name";
-static const char valuestring[]="value";
-static const char numberstring[]="number";
 static const char srcstring[]="src";
 
 static const char *failed_to_parse_XML=ERR_MSG("DjVuMessage.failed_to_parse_XML");
@@ -146,7 +142,6 @@ static const char RootDjVuDir[] ="/etc/DjVu/";  // global last resort
 static const char DjVuEnv[] = "DJVU_CONFIG_DIR";
 
 //  The name of the message file
-static const char DjVuMessageFileName[] = "message";
 static const char MessageFile[]="messages.xml";
 static const char LanguageFile[]="languages.xml";
 
@@ -286,11 +281,9 @@ DjVuMessage::GetProfilePaths(void)
     GMap<GUTF8String,void *> pathsmap;
     GList<GURL> paths;
     GURL path;
-#if !defined(UNDER_CE)
     const GUTF8String envp(GOS::getenv(DjVuEnv));
     if(envp.length())
       appendPath(GURL::Filename::UTF8(envp),pathsmap,paths);
-#endif
 #if defined(WIN32) || defined(UNIX)
     GURL mpath(GetModulePath());
     if(!mpath.is_empty() && mpath.is_dir())
@@ -303,6 +296,8 @@ DjVuMessage::GetProfilePaths(void)
       appendPath(GURL::UTF8(ModuleDjVuDir,mpath),pathsmap,paths);
       mpath=mpath.base();
       appendPath(GURL::UTF8(ModuleDjVuDir,mpath),pathsmap,paths);
+      mpath=mpath.base();
+      appendPath(GURL::UTF8(ModuleDjVuDir,mpath),pathsmap,paths);
     }
 #endif
 #if defined(AUTOCONF)
@@ -333,7 +328,6 @@ DjVuMessage::GetProfilePaths(void)
     pathsmap.empty();
 
     GPosition pos;
-#ifndef UNDER_CE
     GList< GMap<GUTF8String,GP<lt_XMLTags> > > localemaps;
     for(pos=paths;pos;++pos)
     {
@@ -433,13 +427,10 @@ DjVuMessage::GetProfilePaths(void)
     }
     for(pos=localepaths;pos;++pos)
       appendPath(localepaths[pos],pathsmap,realpaths);
-#endif
     for(pos=paths;pos;++pos)
       appendPath(paths[pos],pathsmap,realpaths);
-#ifndef UNDER_CE
     for(pos=osilocalepaths;pos;++pos)
       appendPath(osilocalepaths[pos],pathsmap,realpaths);
-#endif
     for(pos=paths;pos;++pos)
       {
         path=GURL::UTF8(opensourcedir,paths[pos]);
