@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: GOS.cpp,v 1.10 2003/11/07 22:08:21 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: GOS.cpp,v 1.12 2005/04/27 16:34:13 leonb Exp $
+// $Name: release_3_5_16 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -75,9 +75,7 @@
 #ifdef WIN32
 # include <atlbase.h>
 # include <windows.h>
-# ifndef UNDER_CE
-#  include <direct.h>
-# endif
+# include <direct.h>
 #endif
 
 #ifdef OS2
@@ -141,12 +139,10 @@ namespace DJVU {
 char *
 strerror(int errno)
 {
-#ifndef UNDER_CE
   extern int sys_nerr;
   extern char *sys_errlist[];
   if (errno>0 && errno<sys_nerr) 
     return sys_errlist[errno];
-#endif
   return "unknown stdio error";
 }
 #endif
@@ -154,26 +150,10 @@ strerror(int errno)
 
 static const char slash='/';
 static const char percent='%';
-static const char localhostspec1[] = "//localhost/";
 static const char backslash='\\';
 static const char colon=':';
-static const char localhost[] = "localhost";
-static const char localhostspec2[] = "///";
-static const char filespec[] = "file:";
 static const char dot='.';
 static const char nillchar=0;
-#if defined(UNIX)
-  static const char tilde='~';
-  static const char root[] = "/";
-#elif defined(WIN32) || defined(OS2)
-  static const char root[] = "\\";
-#elif defined(macintosh)
-  static char const * const root = &nillchar; 
-#else
-# error "Define something here for your operating system"
-#endif
-
-
 
 
 // -----------------------------------------
@@ -349,8 +329,6 @@ GOS::cwd(const GUTF8String &dirname)
   if (!result)
     G_THROW(errmsg());
   return GNativeString(result).getNative2UTF8();//MBCS cvt
-#elif defined(UNDER_CE)
-  return GUTF8String(dot) ;
 #elif defined (WIN32)
   char drv[2];
   if (dirname.length() && _chdir(dirname.getUTF82Native())==-1)//MBCS cvt
@@ -370,7 +348,6 @@ GUTF8String
 GOS::getenv(const GUTF8String &name)
 {
   GUTF8String retval;
-#ifndef UNDER_CE
   if(name.length())
   {
     const char *env=::getenv(name.getUTF82Native());
@@ -379,7 +356,6 @@ GOS::getenv(const GUTF8String &name)
       retval=GNativeString(env);
     }
   }
-#endif
   return retval;
 }
 
