@@ -51,8 +51,8 @@
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
 // 
-// $Id: debug.cpp,v 1.12 2004/04/17 23:56:11 leonb Exp $
-// $Name: release_3_5_14 $
+// $Id: debug.cpp,v 1.14 2005/05/27 14:26:00 leonb Exp $
+// $Name: release_3_5_16 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -62,20 +62,19 @@
 #endif
 
 #include "debug.h"
-#include "GString.h"
-#include "ByteStream.h"
-#include "GURL.h"
 
 #if ( DEBUGLVL > 0 )
 
 #include "GThreads.h"
 #include "GContainer.h"
+#include "GString.h"
+#include "GString.h"
+#include "ByteStream.h"
+#include "GURL.h"
+
 #include <stdarg.h>
 #include <stdio.h>
-#ifndef UNDER_CE
 #include <errno.h>
-#endif
-
 
 #ifdef HAVE_NAMESPACES
 namespace DJVU {
@@ -266,6 +265,15 @@ DjVuDebug& DjVuDebug::operator<<(const unsigned char * const ptr)
   return operator<<( (const char *) ptr );
 }
 
+DjVuDebug& DjVuDebug::operator<<(const GUTF8String &ptr)
+{
+  GUTF8String buffer(ptr);
+  if(buffer.length() > 255)
+    buffer=buffer.substr(0,252)+"...";
+  format("%s", (const char *)buffer);
+  return *this; 
+}
+
 DjVuDebugIndent::DjVuDebugIndent(int inc)
   : inc(inc)
 {
